@@ -6,7 +6,8 @@
 #include "ptypes.h"
 #include "primality.h"
 #include "mulmod.h"
-#define FUNC_gcd_ui
+#define FUNC_gcd_ui 1
+#define FUNC_is_perfect_square
 #include "util.h"
 
 /* Primality related functions, including Montgomery math */
@@ -17,7 +18,7 @@ static const UV mr_bases_const2[1] = {2};
   Code inside USE_MONT_PRIMALITY is Montgomery math and efficient M-R from
   Wojciech Izykowski.  See:  https://github.com/wizykowski/miller-rabin
 
-Copyright (c) 2013, Wojciech Izykowski
+Copyright (c) 2013-2014, Wojciech Izykowski
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -45,16 +46,14 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 static INLINE uint64_t mont_prod64(uint64_t a, uint64_t b, uint64_t n, uint64_t npi)
 {
   uint64_t t_hi, t_lo, m, mn_hi, mn_lo, u;
-  int carry;
   /* t_hi * 2^64 + t_lo = a*b */
   asm("mulq %3" : "=a"(t_lo), "=d"(t_hi) : "a"(a), "rm"(b));
   m = t_lo * npi;
   /* mn_hi * 2^64 + mn_lo = m*n */
   asm("mulq %3" : "=a"(mn_lo), "=d"(mn_hi) : "a"(m), "rm"(n));
-  carry = t_lo + mn_lo < t_lo ? 1 : 0;
-  u = t_hi + mn_hi + carry;
-  if (u < t_hi) return u-n;
-  return u >= n ? u-n : u;
+  u = t_hi + mn_hi;
+  if (t_lo + mn_lo < t_lo) u++;
+  return (u < t_hi || u >= n)  ?  u-n  :  u;
 }
 #define mont_square64(a, n, npi)  mont_prod64(a, a, n, npi)
 static INLINE UV mont_powmod64(uint64_t a, uint64_t k, uint64_t one, uint64_t n, uint64_t npi)
@@ -67,28 +66,44 @@ static INLINE UV mont_powmod64(uint64_t a, uint64_t k, uint64_t one, uint64_t n,
   }
   return t;
 }
+/* Returns -a^-1 mod 2^64.  From B. Arazi "On Primality Testing Using Purely
+ * Divisionless Operations", Computer Journal (1994) 37 (3): 219-222, Proc 5 */
 static INLINE uint64_t modular_inverse64(const uint64_t a)
 {
-  uint64_t u,x,w,z,q;
-
-  x = 1; z = a;
-
-  q = (-z)/z + 1;  /* = 2^64 / z                 */
-  u = - q;         /* = -q * x                   */
-  w = - q * z;     /* = b - q * z = 2^64 - q * z */
-
-  /* after first iteration all variables are 64-bit */
-
-  while (w) {
-    if (w < z) {
-      q = u; u = x; x = q; /* swap(u, x) */
-      q = w; w = z; z = q; /* swap(w, z) */
+#if 0
+  uint64_t S = 1, J = 0;
+  int i;
+  for (i = 0; i < 64; i++) {
+    if (S & 1) {
+      J |= (1ULL << i);
+      S += a;
     }
-    q = w / z;
-    u -= q * x;
-    w -= q * z;
+    S >>= 1;
   }
-  return x;
+#else
+  /* 2 at a time. */
+  uint64_t S = 1, J = 0;
+  const uint64_t a1p = (a>>1)+1, a2 = a>>2, a2p = a2+1;
+  int i;
+  if (a & 2) {
+    const uint64_t tab[4] = { 0,  a2p,  a1p,  a1p+a2p };
+    for (i = 0; i < 32; i++) {
+      uint64_t S2 = S >> 2;
+      int index = S & 3;
+      J |= (uint64_t)index << (2 * i);
+      S = S2 + tab[index];
+    }
+  } else {
+    const uint64_t tab[4] = { 0,  a1p+a2,  a1p,  a2p };
+    for (i = 0; i < 32; i++) {
+      uint64_t S2 = S >> 2;
+      int index = S & 3;
+      J |= (uint64_t)((4-index)&3) << (2 * i);
+      S = S2 + tab[index];
+    }
+  }
+#endif
+  return J;
 }
 static INLINE uint64_t compute_modn64(const uint64_t n)
 {
@@ -117,7 +132,7 @@ static INLINE uint64_t compute_2_65_mod_n(const uint64_t n, const uint64_t modn)
 static int monty_mr64(const uint64_t n, const UV* bases, int cnt)
 {
   int i, j, t;
-  const uint64_t npi = modular_inverse64(-((int64_t)n));
+  const uint64_t npi = modular_inverse64(n);
   const uint64_t r = compute_modn64(n);
   uint64_t u = n - 1;
   const uint64_t nr = n - r;
@@ -141,33 +156,12 @@ static int monty_mr64(const uint64_t n, const UV* bases, int cnt)
   }
   return 1;
 }
-#else
-#if defined(__GNUC__)
-#define UNUSEDVAR __attribute__ ((unused))
-#else
-#define UNUSEDVAR
-#endif
-static int monty_mr64(const UNUSEDVAR uint64_t n, const UNUSEDVAR UV* bases, int UNUSEDVAR cnt)
-{ croak("configuration error in primality.c"); }
 #endif
 /******************************************************************************/
 
 
 
 
-/* Helper functions */
-static int is_perfect_square(UV n, UV* sqrtn)
-{
-  UV m;
-  m = n & 127;
-  if ((m*0x8bc40d7d) & (m*0xa1e2f5d1) & 0x14020a)  return 0;
-  m = isqrt(n);
-  if (n != (m*m))
-    return 0;
-  if (sqrtn != 0)
-    *sqrtn = m;
-  return 1;
-}
 static int jacobi_iu(IV in, UV m) {
   int j = 1;
   UV n = (in < 0) ? -in : in;
@@ -211,17 +205,18 @@ int _XS_is_pseudoprime(UV const n, UV a)
  */
 int _XS_miller_rabin(UV const n, const UV *bases, int nbases)
 {
+#if USE_MONT_PRIMALITY
+  MPUassert(n > 3, "MR called with n <= 3");
+  if ((n & 1) == 0) return 0;
+  return monty_mr64((uint64_t)n, bases, nbases);
+#else
   UV d = n-1;
   int b, r, s = 0;
 
   MPUassert(n > 3, "MR called with n <= 3");
   if ((n & 1) == 0) return 0;
 
-  if (USE_MONT_PRIMALITY && n >= UVCONST(4294967295))
-    return monty_mr64((uint64_t)n, bases, nbases);
-
   while (!(d&1)) {  s++;  d >>= 1;  }
-
   for (b = 0; b < nbases; b++) {
     UV x, a = bases[b];
     if (a < 2)  croak("Base %"UVuf" is invalid", a);
@@ -242,6 +237,7 @@ int _XS_miller_rabin(UV const n, const UV *bases, int nbases)
     if (r >= s)  return 0;
   }
   return 1;
+#endif
 }
 
 int _XS_BPSW(UV const n)
@@ -253,11 +249,8 @@ int _XS_BPSW(UV const n)
   return    _XS_miller_rabin(n, mr_bases_const2, 1)
          && _XS_is_almost_extra_strong_lucas_pseudoprime(n,1);
 #else
-  if (n < UVCONST(4294967295)) {
-    return    _XS_miller_rabin(n, mr_bases_const2, 1)
-           && _XS_is_almost_extra_strong_lucas_pseudoprime(n,1);
-  } else {
-    const uint64_t npi = modular_inverse64(-((int64_t)n));
+  {
+    const uint64_t npi = modular_inverse64(n);
     const uint64_t montr = compute_modn64(n);
     const uint64_t mont2 = compute_2_65_mod_n(n, montr);
     uint64_t u = n-1;
@@ -291,7 +284,7 @@ int _XS_BPSW(UV const n)
         return 0;
       if (jacobi_iu(D, n) == -1)
         break;
-      if (P == (3+20) && is_perfect_square(n, 0)) return 0;
+      if (P == (3+20) && is_perfect_square(n)) return 0;
       P++;
       if (P > 65535)
         croak("lucas_extrastrong_params: P exceeded 65535");
@@ -308,11 +301,12 @@ int _XS_BPSW(UV const n)
       V = montP;
       { UV v = d; b = 1; while (v >>= 1) b++; }
       while (b-- > 1) {
+        UV T = submod(  mont_prod64(V, W, n, npi),  montP, n);
         if ( (d >> (b-1)) & UVCONST(1) ) {
-          V = submod(  mont_prod64(V, W, n, npi),  montP, n);
+          V = T;
           W = submod(  mont_prod64(W, W, n, npi),  mont2, n);
         } else {
-          W = submod(  mont_prod64(V, W, n, npi),  montP, n);
+          W = T;
           V = submod(  mont_prod64(V, V, n, npi),  mont2, n);
         }
       }
@@ -337,6 +331,7 @@ void lucas_seq(UV* Uret, UV* Vret, UV* Qkret, UV n, IV P, IV Q, UV k)
 {
   UV U, V, b, Dmod, Qmod, Pmod, Qk;
 
+  MPUassert(n > 1, "lucas_sequence:  modulus n must be > 1");
   if (k == 0) {
     *Uret = 0;
     *Vret = 2;
@@ -347,7 +342,13 @@ void lucas_seq(UV* Uret, UV* Vret, UV* Qkret, UV n, IV P, IV Q, UV k)
   Qmod = (Q < 0)  ?  (UV)(Q + (IV)n)  :  (UV)Q;
   Pmod = (P < 0)  ?  (UV)(P + (IV)n)  :  (UV)P;
   Dmod = submod( mulmod(Pmod, Pmod, n), mulmod(4, Qmod, n), n);
-  MPUassert(Dmod != 0, "lucas_seq: D is 0");
+  if (Dmod == 0) {
+    b = Pmod >> 1;
+    *Uret = mulmod(k, powmod(b, k-1, n), n);
+    *Vret = mulmod(2, powmod(b, k, n), n);
+    *Qkret = powmod(Qmod, k, n);
+    return;
+  }
   U = 1;
   V = Pmod;
   Qk = Qmod;
@@ -428,7 +429,7 @@ int _XS_is_lucas_pseudoprime(UV n, int strength)
       if (gcd_ui(Du, n) > 1 && gcd_ui(Du, n) != n) return 0;
       if (jacobi_iu(D, n) == -1)
         break;
-      if (Du == 21 && is_perfect_square(n, 0)) return 0;
+      if (Du == 21 && is_perfect_square(n)) return 0;
       Du += 2;
       sign = -sign;
     }
@@ -442,7 +443,7 @@ int _XS_is_lucas_pseudoprime(UV n, int strength)
       if (gcd_ui(D, n) > 1 && gcd_ui(D, n) != n) return 0;
       if (jacobi_iu(D, n) == -1)
         break;
-      if (P == 21 && is_perfect_square(n, 0)) return 0;
+      if (P == 21 && is_perfect_square(n)) return 0;
       P++;
     }
   }
@@ -454,8 +455,8 @@ int _XS_is_lucas_pseudoprime(UV n, int strength)
     while ( (d & 1) == 0 ) {  s++;  d >>= 1; }
 
 #if USE_MONT_PRIMALITY
-  if (n > UVCONST(4294967295)) {
-    const uint64_t npi = modular_inverse64(-((int64_t)n));
+  {
+    const uint64_t npi = modular_inverse64(n);
     const uint64_t mont1 = compute_modn64(n);
     const uint64_t mont2 = compute_2_65_mod_n(n, mont1);
     const uint64_t montP = (P == 1) ? mont1
@@ -537,8 +538,7 @@ int _XS_is_lucas_pseudoprime(UV n, int strength)
     }
     return 0;
   }
-#endif
-
+#else
   lucas_seq(&U, &V, &Qk, n, P, Q, d);
 
   if (strength == 0) {
@@ -569,14 +569,17 @@ int _XS_is_lucas_pseudoprime(UV n, int strength)
     }
   }
   return 0;
+#endif
 }
 
 /* A generalization of Pari's shortcut to the extra-strong Lucas test.
+ *
+ * This only calculates and tests V, which means less work, but it does result
+ * in a few more pseudoprimes than the full extra-strong test.
+ *
  * I've added a gcd check at the top, which needs to be done and also results
  * in fewer pseudoprimes.  Pari always does trial division to 100 first so
- * is unlikely to come up there.  This only calculate V, which can be done
- * faster, but that means we have more pseudoprimes than the standard
- * extra-strong test.
+ * is unlikely to come up there.
  *
  * increment:  1 for Baillie OEIS, 2 for Pari.
  *
@@ -590,7 +593,7 @@ int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment)
   if (n < 7) return (n == 2 || n == 3 || n == 5);
   if ((n % 2) == 0 || n == UV_MAX) return 0;
   if (increment < 1 || increment > 256)
-    croak("Invalid lucas paramater increment: %"UVuf"\n", increment);
+    croak("Invalid lucas parameter increment: %"UVuf"\n", increment);
 
   P = 3;
   while (1) {
@@ -600,7 +603,7 @@ int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment)
       return 0;
     if (jacobi_iu(D, n) == -1)
       break;
-    if (P == (3+20*increment) && is_perfect_square(n, 0)) return 0;
+    if (P == (3+20*increment) && is_perfect_square(n)) return 0;
     P += increment;
     if (P > 65535)
       croak("lucas_extrastrong_params: P exceeded 65535");
@@ -613,19 +616,20 @@ int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment)
   { UV v = d; b = 0; while (v >>= 1) b++; }
 
 #if USE_MONT_PRIMALITY
-  if (n > UVCONST(4294967295)) {
-    const uint64_t npi = modular_inverse64(-((int64_t)n));
+  {
+    const uint64_t npi = modular_inverse64(n);
     const uint64_t montr = compute_modn64(n);
     const uint64_t mont2 = compute_2_65_mod_n(n, montr);
     const uint64_t montP = compute_a_times_2_64_mod_n(P, n, montr);
     W = submod(  mont_prod64( montP, montP, n, npi),  mont2, n);
     V = montP;
     while (b--) {
+      UV T = submod(  mont_prod64(V, W, n, npi),  montP, n);
       if ( (d >> b) & UVCONST(1) ) {
-        V = submod(  mont_prod64(V, W, n, npi),  montP, n);
+        V = T;
         W = submod(  mont_prod64(W, W, n, npi),  mont2, n);
       } else {
-        W = submod(  mont_prod64(V, W, n, npi),  montP, n);
+        W = T;
         V = submod(  mont_prod64(V, V, n, npi),  mont2, n);
       }
     }
@@ -641,15 +645,16 @@ int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment)
     }
     return 0;
   }
-#endif
+#else
   W = mulsubmod(P, P, 2, n);
   V = P;
   while (b--) {
+    UV T = mulsubmod(V, W, P, n);
     if ( (d >> b) & UVCONST(1) ) {
-      V = mulsubmod(V, W, P, n);
+      V = T;
       W = mulsubmod(W, W, 2, n);
     } else {
-      W = mulsubmod(V, W, P, n);
+      W = T;
       V = mulsubmod(V, V, 2, n);
     }
   }
@@ -663,6 +668,7 @@ int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment)
       return 0;
   }
   return 0;
+#endif
 }
 
 /*
@@ -679,12 +685,12 @@ int _XS_is_almost_extra_strong_lucas_pseudoprime(UV n, UV increment)
 int _XS_is_frobenius_underwood_pseudoprime(UV n)
 {
   int bit;
-  UV x, result, a, b, np1, len, t1, t2, na;
+  UV x, result, a, b, np1, len, t1;
   IV t;
 
   if (n < 7) return (n == 2 || n == 3 || n == 5);
   if ((n % 2) == 0 || n == UV_MAX) return 0;
-  if (is_perfect_square(n,0)) return 0;
+  if (is_perfect_square(n)) return 0;
 
   x = 0;
   t = -1;
@@ -696,8 +702,8 @@ int _XS_is_frobenius_underwood_pseudoprime(UV n)
   { UV v = np1; len = 1;  while (v >>= 1) len++; }
 
 #if USE_MONT_PRIMALITY
-  if (n > UVCONST(4294967295)) {
-    const uint64_t npi = modular_inverse64(-((int64_t)n));
+  {
+    const uint64_t npi = modular_inverse64(n);
     const uint64_t mont1 = compute_modn64(n);
     const uint64_t mont2 = compute_2_65_mod_n(n, mont1);
     const uint64_t mont5 = compute_a_times_2_64_mod_n(5, n, mont1);
@@ -709,82 +715,58 @@ int _XS_is_frobenius_underwood_pseudoprime(UV n)
     if (x == 0) {
       result = mont5;
       for (bit = len-2; bit >= 0; bit--) {
-        t2 = addmod(b, b, n);
-        na = mont_prod64(a, t2, n, npi);
-        t1 = addmod(b, a, n);
-        t2 = submod(b, a, n);
-        b = mont_prod64(t1, t2, n, npi);
-        a = na;
+        t1 = addmod(b, b, n);
+        b = mont_prod64(submod(b, a, n), addmod(b, a, n), n, npi);
+        a = mont_prod64(a, t1, n, npi);
         if ( (np1 >> bit) & UVCONST(1) ) {
-          t1 = addmod(a, a, n);
-          na = addmod(t1, b, n);
-          t1 = addmod(b, b, n);
-          b = submod(t1, a, n);
-          a = na;
+          t1 = b;
+          b = submod( addmod(b, b, n), a, n);
+          a = addmod( addmod(a, a, n), t1, n);
         }
       }
     } else {
       UV multiplier = addmod(x, mont2, n);
       result = addmod( addmod(x, x, n), mont5, n);
       for (bit = len-2; bit >= 0; bit--) {
-        t1 = mont_prod64(a, x, n, npi);
-        t2 = addmod(b, b, n);
-        t1 = addmod(t1, t2, n);
-        na = mont_prod64(a, t1, n, npi);
-        t1 = addmod(b, a, n);
-        t2 = submod(b, a, n);
-        b = mont_prod64(t1, t2, n, npi);
-        a = na;
+        t1 = addmod( mont_prod64(a, x, n, npi), addmod(b, b, n), n);
+        b = mont_prod64(submod(b, a, n), addmod(b, a, n), n, npi);
+        a = mont_prod64(a, t1, n, npi);
         if ( (np1 >> bit) & UVCONST(1) ) {
-          t1 = mont_prod64(a, multiplier, n, npi);
-          na = addmod(t1, b, n);
-          t1 = addmod(b, b, n);
-          b = submod(t1, a, n);
-          a = na;
+          t1 = b;
+          b = submod( addmod(b, b, n), a, n);
+          a = addmod( mont_prod64(a, multiplier, n, npi), t1, n);
         }
       }
     }
     return (a == 0 && b == result);
   }
-#endif
+#else
   a = 1;
   b = 2;
 
   if (x == 0) {
     result = 5;
     for (bit = len-2; bit >= 0; bit--) {
-      t2 = addmod(b, b, n);
-      na = mulmod(a, t2, n);
-      t1 = addmod(b, a, n);
-      t2 = submod(b, a, n);
-      b = mulmod(t1, t2, n);
-      a = na;
+      t1 = addmod(b, b, n);
+      b = mulmod( submod(b, a, n), addmod(b, a, n), n);
+      a = mulmod(a, t1, n);
       if ( (np1 >> bit) & UVCONST(1) ) {
-        t1 = addmod(a, a, n);
-        na = addmod(t1, b, n);
-        t1 = addmod(b, b, n);
-        b = submod(t1, a, n);
-        a = na;
+        t1 = b;
+        b = submod( addmod(b, b, n), a, n);
+        a = addmod( addmod(a, a, n), t1, n);
       }
     }
   } else {
     UV multiplier = addmod(x, 2, n);
     result = addmod( addmod(x, x, n), 5, n);
     for (bit = len-2; bit >= 0; bit--) {
-      t1 = mulmod(a, x, n);
-      t2 = addmod(b, b, n);
-      t1 = addmod(t1, t2, n);
-      na = mulmod(a, t1, n);
-      t1 = addmod(b, a, n);
-      t2 = submod(b, a, n);
-      b = mulmod(t1, t2, n);
-      a = na;
+      t1 = addmod( mulmod(a, x, n), addmod(b, b, n), n);
+      b = mulmod(submod(b, a, n), addmod(b, a, n), n);
+      a = mulmod(a, t1, n);
       if ( (np1 >> bit) & UVCONST(1) ) {
-        t1 = mulmod(a, multiplier, n);
-        na = addmod(t1, b, n);
-        t1 = addmod(b, b, n);
-        b = submod(t1, a, n);
-        a = na;
+        t1 = b;
+        b = submod( addmod(b, b, n), a, n);
+        a = addmod( mulmod(a, multiplier, n), t1, n);
       }
     }
   }
@@ -793,6 +775,7 @@ int _XS_is_frobenius_underwood_pseudoprime(UV n)
   if (a == 0 && b == result)
     return 1;
   return 0;
+#endif
 }
 
 
@@ -813,7 +796,7 @@ static const UV mr_bases_large_3[3] = { UVCONST(  4230279247111683200 ),
 static const UV mr_bases_large_7[7] = { 2, 325, 9375, 28178, 450775, 9780504, 1795265022 };
 #endif
 
-int _XS_is_prob_prime(UV n)
+int is_prob_prime(UV n)
 {
   int ret;
 
